import unittest

import ecanvci


class TestECanVci(unittest.TestCase):
    def test_is_string(self):
        device = ecanvci.VciDevice(config=ecanvci.VciInitConfig(Mode=2))    # Mode=2 means CAN bus self-loop
        obj = ecanvci.VciCanObj()
        for i in range(5):
            obj.Data[0] = i
            device.transmit(obj)
            ret, recv_msg = device.receive(waittime=-1)
            # print(ret, recv_msg.ID, bytearray(recv_msg.Data), bytearray(obj.Data))
            self.assertTrue(ret == 1)
            self.assertTrue(recv_msg.ID == obj.ID)
            self.assertTrue(bytearray(recv_msg.Data) == bytearray(obj.Data))
        device.close()


if __name__ == '__main__':
    unittest.main()


from .ecanvci import VciCanObj
import time


class CanRecord(bytearray):
    def add(self, msg:VciCanObj, current=None):
        try:
            current_ms = int(current * 1000)
        except:
            current_ms = int(time.time() * 1000)
        self += bytearray(msg)
        self[-3] = current_ms  & 0xFF
        self[-2] = (current_ms >> 8)  & 0xFF
        self[-1] = (current_ms >> 16)  & 0xFF
        # print(f"{msg.ID}, len(record)={len(self)}")



from .ecanvci import VciInitConfig, VciDevice, VciCanObj
from ctypes import byref
import threading
import queue
import signal
import time
import logging

logging.basicConfig(level=logging.NOTSET, format='%(asctime)s %(message)s', datefmt='%H:%M:%S')


class ECanVciHelper:
    def __init__(self, config=VciInitConfig(), log=False):
        def signal_handler(sig, frame):
            logging.info('You pressed Ctrl+C!')
            self.stop()

        signal.signal(signal.SIGINT, signal_handler)

        self.log = log
        self.device = VciDevice(config=config)
        self.evt = threading.Event()
        self.q = queue.Queue()
        self.t = threading.Thread(target=self.thread_main, daemon=True)
        self.out_q = queue.Queue()
        self.t.start()

    def thread_main(self):
        while not self.evt.is_set():
            recv_msg = VciCanObj(SendType=0)
            while self.device.receive_by_ref(byref(recv_msg), 1, 0):
                if self.log:
                    logging.info(f">>> {hex(recv_msg.ID)} {list(recv_msg.Data[:recv_msg.DataLen])}")
                self.out_q.put(recv_msg)
            try:
                msg = self.q.get_nowait()
                assert(self.device.transmit_by_ref(byref(msg), 1) == 1)
                if self.log:
                    logging.info(f"<<< {hex(msg.ID)} {list(msg.Data[:msg.DataLen])}")
            except queue.Empty:
                # time.sleep(0.001)
                pass
        logging.info("thread exit")

    def stop(self):
        logging.info("stopping")
        self.evt.set()
        while self.t.is_alive():
            self.t.join(timeout=1)
        self.device.close()

    def send(self, id=0, data=(0,)*8):
        assert(not self.evt.is_set())
        self.q.put(VciCanObj(ID=id, Data=data, DataLen=len(data)))

    def send_vcicanobj(self, obj):
        assert(not self.evt.is_set())
        self.q.put(obj)

    def recv(self, block=True, timeout=None):
        if self.evt.is_set():
            return None
        return self.out_q.get(block, timeout)

    def clear_recv_queue(self):
        try:
            while (not self.evt.is_set()) and self.out_q.get_nowait():
                pass
        except:
            pass

    def wait_for_send(self):
        while (not self.evt.is_set()) and (not self.q.empty()):
            time.sleep(0.1)

    def keep_running(self):
        while self.t.is_alive():
            self.t.join(timeout=0.1)


'''
def _simple_demo():
    import time

    config=VciInitConfig(Mode=2)
    can = ECanVciHelper(config=config)    # Mode=2 means CAN bus self-loop

    logging.info("press CTRL+C to exit")

    for i in range(5):
        can.send(id=i)
        time.sleep(1)

    for i in range(5):
        recv_msg = can.recv()
        logging.info(recv_msg.ID, bytearray(recv_msg.Data))

    can.keep_running()

    logging.info("main exit")


if __name__ == "__main__":
    _simple_demo()
'''


from ctypes import Structure, LittleEndianStructure, memmove, addressof, sizeof
import sys

ECANVCI_TIMING = {5: (0xBF, 0xFF),
                  10: (0x31, 0x1C),
                  20: (0x18, 0x1C),
                  40: (0x87, 0xFF),
                  50: (0x09, 0x1C),
                  80: (0x83, 0xFF),
                  100: (0x04, 0x1C), 
                  125: (0x03, 0x1C), 
                  200: (0x81, 0xFA), 
                  250: (0x01, 0x1C),
                  400: (0x80, 0xFA), 
                  500: (0x00, 0x1C),
                  666: (0x80, 0xB6), 
                  800: (0x00, 0x16), 
                  1000: (0x00, 0x14)}

# ECANVCI_TIMING0_100KBPS, ECANVCI_TIMING1_100KBPS = ECANVCI_TIMING[100]
# ECANVCI_TIMING0_250KBPS, ECANVCI_TIMING1_250KBPS = ECANVCI_TIMING[250]
# ECANVCI_TIMING0_500KBPS, ECANVCI_TIMING1_500KBPS = ECANVCI_TIMING[500]

this_module = sys.modules[__name__]
for kbps, timings in ECANVCI_TIMING.items():
    setattr(this_module, f"ECANVCI_TIMING0_{kbps}KBPS", timings[0])
    setattr(this_module, f"ECANVCI_TIMING1_{kbps}KBPS", timings[1])


class StructureEx(Structure):
    '''Add easy-to-use method to set values to ctypes.structure
    '''
    def from_iterable(self, iter):
        assert(len(iter) == sizeof(self))
        for (field_name, field_type), value in zip(self._fields_, iter):
            setattr(self, field_name, value)

    def from_array(self, arr):
        memmove(addressof(self), addressof(arr), sizeof(self))

        
class LEStructureEx(LittleEndianStructure):
    '''Add easy-to-use method to set values to ctypes.structure
    '''
    def from_iterable(self, iter):
        assert(len(iter) == sizeof(self))
        for (field_name, field_type), value in zip(self._fields_, iter):
            setattr(self, field_name, value)

    def from_array(self, arr):
        memmove(addressof(self), addressof(arr), sizeof(self))